//
//  File.swift
//  Measured Blood Lost
//
//  Created by Josh Birnholz on 4/13/16.
//  Copyright © 2016 Josh Birnholz. All rights reserved.
//

import UIKit

class AddNewWeightViewController : UITableViewController, UITextFieldDelegate {
	
	var weight: Weight?
	var categoryOperatingRoom = false
	var categoryVaginalDelivery = false
	var name: String?
	var weightInGrams: Int?
	var allWeightsIndex: Int?
	
	@IBOutlet weak var nameTextField: UITextField!
	
	@IBOutlet weak var weightTextField: UITextField!
	
	
	@IBOutlet weak var operatingRoomCell: UITableViewCell!
	
	@IBOutlet weak var vaginalDeliveryCell: UITableViewCell!
	
	override func viewDidLoad() {
		if name != nil {
			nameTextField.text = name!
		}
		
		if weightInGrams != nil {
			weightTextField.text = String(weightInGrams!)
		}
		
		operatingRoomCell.accessoryType = categoryOperatingRoom ? .Checkmark : .None
		vaginalDeliveryCell.accessoryType = categoryVaginalDelivery ? .Checkmark : .None
		
		navigationItem.rightBarButtonItem?.enabled = savingShouldBeEnabled()
		
		nameTextField.delegate = self
		weightTextField.delegate = self
		
	}
	
	override func tableView(tableView: UITableView, willSelectRowAtIndexPath indexPath: NSIndexPath) -> NSIndexPath? {
		if indexPath.section == 2 {
			return indexPath
		} else {
			return nil
		}
	}
	
	override func tableView(tableView: UITableView, shouldHighlightRowAtIndexPath indexPath: NSIndexPath) -> Bool {
		if indexPath.section == 2 {
			return true
		} else {
			return false
		}
	}
	
	override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
		switch indexPath.section {
		case 0:
			nameTextField.becomeFirstResponder()
		case 1:
			weightTextField.becomeFirstResponder()
		default:
			
			let cell = tableView.cellForRowAtIndexPath(NSIndexPath(forRow: indexPath.row, inSection: indexPath.section))
			
			switch indexPath.row {
			case 0:
				if categoryOperatingRoom {
					categoryOperatingRoom = false
					cell?.accessoryType = .None
					tableView.deselectRowAtIndexPath(indexPath, animated: true)
					print("categoryOperatingRoom was true, is now false")
				} else {
					categoryOperatingRoom = true
					cell?.accessoryType = .Checkmark
					tableView.deselectRowAtIndexPath(indexPath, animated: true)
					print("categoryOperatingRoom was false, is now true")
				}
			case 1:
				if categoryVaginalDelivery {
					categoryVaginalDelivery = false
					cell?.accessoryType = .None
					tableView.deselectRowAtIndexPath(indexPath, animated: true)
					print("categoryVaginalDelivery was true, is now false")
				} else {
					categoryVaginalDelivery = true
					cell?.accessoryType = .Checkmark
					tableView.deselectRowAtIndexPath(indexPath, animated: true)
					print("categoryVaginalDeliverywas false is now true")
				}
			default: break
			}
			navigationItem.rightBarButtonItem?.enabled = savingShouldBeEnabled()
		}
	}
	
	func calculateCategory() -> Category? {
		if categoryOperatingRoom && categoryVaginalDelivery {
			return nil
		}
		else if categoryOperatingRoom {
			return .OperatingRoom
		}
		else {
			return .VaginalDelivery
		}
	}
	
	@IBAction func nameTextFieldEditingDidEnd(sender: AnyObject) {
		navigationItem.rightBarButtonItem?.enabled = savingShouldBeEnabled()
	}
	
	@IBAction func weightTextFieldEditingDidEnd(sender: AnyObject) {
		weightTextField.text!.numericize()
		navigationItem.rightBarButtonItem?.enabled = savingShouldBeEnabled()
	}
	
	func savingShouldBeEnabled() -> Bool {
		return Int(weightTextField.text!) > 0 && nameTextField.text! != "" && (categoryOperatingRoom || categoryVaginalDelivery)
	}
	
	@IBOutlet weak var saveButton: UIBarButtonItem!
	
	override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
		if saveButton === sender {
			if let category = calculateCategory() {
				weight = Weight(name: nameTextField.text!, weightInGrams: Int(weightTextField.text!)!, category: category)
			} else {
				weight = Weight(name: nameTextField.text!, weightInGrams: Int(weightTextField.text!)!, category: nil)
			}
			
		}
	}
	
	func textFieldShouldReturn(textField: UITextField) -> Bool {   //delegate method
		textField.resignFirstResponder()
		if textField == nameTextField {
			weightTextField.becomeFirstResponder()
		}
		
		return true
	}
	
	
}