//
//  AppDelegate.swift
//  Measured Blood Lost
//
//  Created by Josh Birnholz on 4/11/16.
//  Copyright © 2016 Josh Birnholz. All rights reserved.
//

import UIKit

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {
	
	var window: UIWindow?
	
	
	func application(application: UIApplication, didFinishLaunchingWithOptions launchOptions: [NSObject: AnyObject]?) -> Bool {
		// Override point for customization after application launch.
		
		let userDefaults = NSUserDefaults.standardUserDefaults()
		
		if userDefaults.valueForKey("archivedWeights") == nil {
			
			let archivedWeights: NSData
			
			if let weights = downloadWeightsFromServer() {
				archivedWeights = weights.archived
				print("Downloading default weights from server.")
			} else {
				archivedWeights = defaultWeights().archived
				print("Using stored default weights.")
			}
			
			userDefaults.setObject(archivedWeights, forKey: "archivedWeights")
			userDefaults.synchronize()
			
			print("Archived weights to user defaults.")
			
		}
		
		return true
	}
	
	func application(application: UIApplication, openURL url: NSURL, sourceApplication: String?, annotation: AnyObject) -> Bool {
		//url contains a URL to the file your app shall open
		
		//In my EXAMPLE I would want to read the file as a dictionary
		let archivedWeights = NSData(contentsOfURL: url)
		
		let unarchivedWeights = NSKeyedUnarchiver.unarchiveObjectWithData(archivedWeights!) as? [Weight]
		
		if unarchivedWeights == nil {
			
			presentSimpleAlert("Could not load weights", message: "The file \"\(url.lastPathComponent!)\" is invalid.")
			
			return false
			
		} else {
			
			let alertController = UIAlertController(title: "Load Weights", message: "Do you want to replace the list of weights with the ones from the file \"\(url.lastPathComponent!)\"?", preferredStyle: .Alert)
			
			let cancelAction = UIAlertAction(title: "Don't Load", style: UIAlertActionStyle.Default) { (action) in
				// Do nothing
			}
			alertController.addAction(cancelAction)
			
			let loadAction = UIAlertAction(title: "Load", style: UIAlertActionStyle.Destructive) { (actinon) in
				
				let userDefaults = NSUserDefaults()
				userDefaults.setObject(archivedWeights, forKey: "archivedWeights")
				userDefaults.synchronize()
				
				print("Archived default weights to user defaults.")
				
				presentSimpleAlert(nil, message: "Loaded weights successfully.")
				
			}
			alertController.addAction(loadAction)
			
			alertController.show(true)
			
			
		}
		
		return true
		
	}

	
	func applicationWillResignActive(application: UIApplication) {
		// Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
		// Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
	}
	
	func applicationDidEnterBackground(application: UIApplication) {
		// Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
		// If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
	}
	
	func applicationWillEnterForeground(application: UIApplication) {
		// Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
	}
	
	func applicationDidBecomeActive(application: UIApplication) {
		// Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
	}
	
	func applicationWillTerminate(application: UIApplication) {
		// Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
	}
	
	
}

