//
//  ViewController.swift
//  Measured Blood Lost
//
//  Created by Josh Birnholz on 4/11/16.
//  Copyright © 2016 Josh Birnholz. All rights reserved.
//

import UIKit

enum ValueType {
	case IrrigationFluid
	case BloodFromCanister
}

class MainViewController: UITableViewController, WeightCellDelegate, ValueCellDelegate {
	
	@IBOutlet weak var totalWeightBarButtonItem: UIBarButtonItem!
	private var totalWeightLabel = UILabel(frame: CGRectZero)
	
	@IBOutlet weak var totalWeightTextField: UITextField!
	
	var totalWeight: Int = 0
	var irrigationFluid: Int = 0
	var bloodFromCanister: Int = 0
	
	var weights = [Weight]()
	
	override func viewDidLoad() {
		super.viewDidLoad()
		
		// Do any additional setup after loading the view, typically from a nib.
		
		totalWeightLabel.backgroundColor = UIColor.clearColor()
		totalWeightLabel.textAlignment = .Center
		totalWeightBarButtonItem.customView = totalWeightLabel
		
		navigationItem.leftBarButtonItem = editButtonItem()
		
		updateTotalWeight()
		
	}
	
	override func didReceiveMemoryWarning() {
		super.didReceiveMemoryWarning()
		// Dispose of any resources that can be recreated.
	}
	
	override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
		return 1
	}
 
	override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
		return weights.count + 2
	}
	
	override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath)
		-> UITableViewCell {
			if indexPath.row < weights.count {
				let cell = tableView.dequeueReusableCellWithIdentifier("WeightCell", forIndexPath: indexPath) as! WeightCell
				
				let name = weights[indexPath.row].name
				let weightInGrams = weights[indexPath.row].weightInGrams
				let quantity = weights[indexPath.row].quantity
				
				if let nameLabel = cell.viewWithTag(100) as? UILabel {
					nameLabel.text = "\(name.pluralize(quantity))"
					nameLabel.textColor = quantity == 0 ? UIColor.lightGrayColor() : UIColor.darkTextColor()
				}
				if let weightLabel = cell.viewWithTag(101) as? UILabel {
					weightLabel.text = "-\(weightInGrams * quantity) gm"
				}
				
				if let quantityTextField = cell.viewWithTag(102) as? UITextField {
					quantityTextField.text = quantity == 0 ? "" : String(quantity)
				}
				cell.delegate = self
				
				return cell
			} else {
				let cell = tableView.dequeueReusableCellWithIdentifier("ValueCell", forIndexPath: indexPath) as! ValueCell
				
				let nameLabel = cell.viewWithTag(100) as! UILabel
				let minusPlusLabel = cell.viewWithTag(101) as! UILabel
				let valueTextField = cell.viewWithTag(102) as! UITextField
				
				switch valueTypeForRow(indexPath.row)! {
				case .IrrigationFluid:
					nameLabel.text = "Irrigation fluid"
					valueTextField.text = irrigationFluid == 0 ? "" : String(irrigationFluid)
					nameLabel.textColor = irrigationFluid == 0 ? UIColor.lightGrayColor() : UIColor.darkTextColor()
					minusPlusLabel.text = "-"
				case .BloodFromCanister:
					nameLabel.text = "Blood from canister"
					valueTextField.text = bloodFromCanister == 0 ? "" : String(bloodFromCanister)
					nameLabel.textColor = bloodFromCanister == 0 ? UIColor.lightGrayColor() : UIColor.darkTextColor()
					minusPlusLabel.text = "+"
				}
				
				cell.delegate = self
				
				return cell
			}
	}
	
	func valueTypeForRow(row: Int) -> ValueType? {
		if row < weights.count {
			return nil
		}
		
		return row == weights.count ? .IrrigationFluid : .BloodFromCanister
	}
	
	func rowForSender(sender: AnyObject) -> Int {
		let point = sender.convertPoint(CGPointZero, toView: tableView)
		let indexPath = self.tableView.indexPathForRowAtPoint(point)!
		
		return indexPath.row
	}
	
	func quantityChanged(newQuantity: Int, actionSender: AnyObject) {
		
		weights[rowForSender(actionSender)].quantity = newQuantity
		tableView.reloadData()
		updateTotalWeight()
		
	}
	
	func valueChanged(cell: ValueCell, newValue: Int, actionSender: AnyObject) {
		
		print("Changing value")
		
		switch valueTypeForRow(rowForSender(actionSender))! {
		case .IrrigationFluid:
			irrigationFluid = newValue
		case .BloodFromCanister:
			bloodFromCanister = newValue
		}
		
		tableView.reloadData()
		updateTotalWeight()
		
	}
	
	func addItems(weightsToAdd: [Weight]) {
		
		for weightToAdd in weightsToAdd {
			
			print("weightToAdd: \(weightToAdd.quantity) \(weightToAdd.name.pluralize(weightToAdd.quantity))")
			
			if weightToAdd.quantity > 0 {
				
				var foundIndex: Int?
				var index = 0
				for presentWeight in weights {
					if presentWeight.name == weightToAdd.name {
						foundIndex = index
					}
					index += 1
				}
				
				if foundIndex == nil {
					
					weights.append(weightToAdd)
					let indexPath = NSIndexPath(forRow: weights.count-1, inSection: 0)
					tableView.insertRowsAtIndexPaths([indexPath], withRowAnimation: .Automatic)
					
				} else {
					weights[foundIndex!].quantity += weightToAdd.quantity
					tableView.reloadData()
				}
				
				
			}
		}
		
		print("-- Finished adding weights --")
	}
	
	@IBAction func totalWeightTextFieldEditingBegan(sender: AnyObject) {
		
		if totalWeightTextField.text!.hasSuffix(" gm") {
			totalWeightTextField.text = totalWeightTextField.text!.substringToIndex(totalWeightTextField.text!.endIndex.advancedBy(-3))
		}
		
	}
	
	@IBAction func totalWeightTextFieldEditingEnded(sender: AnyObject) {
		
		totalWeightTextField.text!.numericize()
		
		if totalWeightTextField.text! == "" {
			totalWeight = 0
		} else {
			
			if let totalWeightInt = Int(totalWeightTextField.text!) {
				totalWeight = totalWeightInt
			} else {
				totalWeight = Int.max
			}
			
			totalWeightTextField.text = "\(String(totalWeight)) gm"
			
		}
		updateTotalWeight()
	}
	
	func updateTotalWeight() {
		
		var measuredBloodLost = totalWeight - irrigationFluid + bloodFromCanister
		
		for weight in weights {
			measuredBloodLost = measuredBloodLost - (weight.weightInGrams * weight.quantity)
		}
		
		if measuredBloodLost < 0 {
			totalWeightLabel.textColor = UIColor.redColor()
		} else {
			totalWeightLabel.textColor = UIColor.darkTextColor()
		}
		
		totalWeightLabel.text = "Measured Blood Lost: \(measuredBloodLost) mL"
		totalWeightLabel.sizeToFit()
	}
	
	override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
		return indexPath.row < weights.count ? true : false
	}
	
	override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
		if editingStyle == .Delete {
			// Remove data
			weights.removeAtIndex(indexPath.row)
			tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: UITableViewRowAnimation.Automatic)
		}
		
		updateTotalWeight()
	}
	
	@IBAction func unwindToWeightsList(sender: UIStoryboardSegue) {
		
		if let sourceViewController = sender.sourceViewController as? SelectWeightsViewController {
			let weightsToAdd = sourceViewController.weightsToAdd
			
			addItems(weightsToAdd)
			updateTotalWeight()
		}
	}
	
	@IBOutlet var addButton: UIBarButtonItem!
	@IBOutlet var resetButton: UIBarButtonItem!
	
	@IBAction func clearState() {
		let alertController = UIAlertController(title: nil, message: "Clear all items and values?", preferredStyle: .ActionSheet)
		
		let cancelAction = UIAlertAction(title: "Cancel", style: .Cancel) { (action) in
			// Do nothing
		}
		alertController.addAction(cancelAction)
		
		let destroyAction = UIAlertAction(title: "Clear", style: .Destructive) { (action) in
			print("restoring state")
			
			self.weights.removeAll()
			self.irrigationFluid = 0
			self.bloodFromCanister = 0
			self.totalWeight = 0
			self.totalWeightTextField.text = ""
			
			self.updateTotalWeight()
			
			self.tableView.reloadData()
			
			if self.editing {
				self.setEditing(false, animated: true)
			}
		}
		alertController.addAction(destroyAction)
		
		if let popover = alertController.popoverPresentationController {
			popover.barButtonItem = resetButton
			popover.sourceView = self.view
		}
		
		self.presentViewController(alertController, animated: true) {
			
		}
	}
	
	override func setEditing(editing: Bool, animated: Bool) {
		super.setEditing(editing, animated: animated)
		
		navigationController!.view.endEditing(true)
		
		if self.editing {
			self.navigationItem.setRightBarButtonItem(resetButton, animated: true)
		} else {
			self.navigationItem.setRightBarButtonItem(addButton, animated: true)
		}
	}
}

