//
//  File.swift
//  Measured Blood Lost
//
//  Created by Josh Birnholz on 4/11/16.
//  Copyright © 2016 Josh Birnholz. All rights reserved.
//

import Foundation

enum Category {
	case OperatingRoom
	case VaginalDelivery
}

class Weight : NSObject, NSCoding {
	
	let name: String
	let weightInGrams: Int
	var quantity: Int
	var category: Category? = nil
	
	init(name: String, weightInGrams: Int) {
		self.weightInGrams = weightInGrams
		self.name = name
		quantity = 0
	}
	
	init(name: String, weightInGrams: Int, category: Category?) {
		self.weightInGrams = weightInGrams
		self.name = name
		self.quantity = 0
		self.category = category
	}
	
	init(name: String, weightInGrams: Int, quantity: Int) {
		self.weightInGrams = weightInGrams
		self.name = name
		self.quantity = quantity
	}
	
	required init(coder aDecoder: NSCoder) {
		name = aDecoder.decodeObjectForKey("name") as! String
		weightInGrams = aDecoder.decodeObjectForKey("weightInGrams") as! Int
		quantity = aDecoder.decodeObjectForKey("quantity") as! Int
		
		if let categoryString = aDecoder.decodeObjectForKey("category") as? String {
			if categoryString == "OperatingRoom" {
				category = .OperatingRoom
			} else {
				category = .VaginalDelivery
			}
		} else {
			category = nil
		}
		
	}
	
	func encodeWithCoder(aCoder: NSCoder) {
		aCoder.encodeObject(name, forKey: "name")
		aCoder.encodeObject(weightInGrams, forKey: "weightInGrams")
		aCoder.encodeObject(quantity, forKey: "quantity")
		
		if category == nil {
			aCoder.encodeObject(nil, forKey: "category")
		} else if category == .OperatingRoom {
			aCoder.encodeObject("OperatingRoom", forKey: "category")
		} else {
			aCoder.encodeObject("VaginalDelivery", forKey: "category")
		}
	}
	
}

func defaultWeights() -> (unarchived:[Weight], archived: NSData) {
	let weights = [Weight(name: "Lap", weightInGrams: 22, category: .OperatingRoom),
	               Weight(name: "Lap counting bag", weightInGrams: 25, category: .OperatingRoom),
	               Weight(name: "Sterile green towel", weightInGrams: 90),
	               Weight(name: "Sterile blue towel", weightInGrams: 55, category: .OperatingRoom),
	               Weight(name: "Kick bucket red bag", weightInGrams: 50, category: .OperatingRoom),
	               Weight(name: "Medium red bag", weightInGrams: 70, category: .OperatingRoom),
	               Weight(name: "Large red bag", weightInGrams: 120),
	               Weight(name: "Washcloth", weightInGrams: 30),
	               Weight(name: "Towel", weightInGrams: 192, category: .OperatingRoom),
	               Weight(name: "Blue cloth underpad", weightInGrams: 340),
	               Weight(name: "Regular patient gown", weightInGrams: 340),
	               Weight(name: "XL patient gown", weightInGrams: 498),
	               Weight(name: "Under buttocks drape", weightInGrams: 76, category: .VaginalDelivery),
	               Weight(name: "Mini lap", weightInGrams: 6, category: .VaginalDelivery),
	               Weight(name: "Vag packing", weightInGrams: 16, category: .VaginalDelivery),
	               Weight(name: "Pink peri pad", weightInGrams: 26, category: .VaginalDelivery),
	               Weight(name: "Large white (Capri +)", weightInGrams: 40, category: .VaginalDelivery)
	]
	
	return (weights, NSKeyedArchiver.archivedDataWithRootObject(weights as NSArray))
}

func downloadWeightsFromServer() -> (unarchived:[Weight], archived: NSData)? {
	let url = NSURL(string: "http://josh.birnholz.com/MBL/WeightList.weights")
	
	if let archivedWeights = NSData(contentsOfURL: url!) {
		
		if let unarchivedWeights = NSKeyedUnarchiver.unarchiveObjectWithData(archivedWeights) as? [Weight] {
			return (unarchivedWeights, archivedWeights)
		}
	}
	
	return nil
}

func downloadWeightsWithFallback() -> (unarchived:[Weight], archived: NSData, downloadSuccess: Bool) {
	if let weights = downloadWeightsFromServer() {
		return (weights.unarchived, weights.archived, true)
	} else {
		let weights = defaultWeights()
		return (weights.unarchived, weights.archived, true)
	}
}